package com.zkqy.system.mapper;

import com.zkqy.system.entity.DragTable;
import com.zkqy.system.entity.vo.DragTableVo;

import java.util.List;

/**
 * 动态表格Mapper接口
 * 
 * @author zkqy
 * @date 2023-07-31
 */
public interface DragTableMapper 
{
    /**
     * 查询动态表格
     * 
     * @param tId 动态表格主键
     * @return 动态表格
     */
    DragTable selectDragTableByTId(Long tId);

    /**
     * 查询动态表格列表
     * 
     * @param dragTable 动态表格
     * @return 动态表格集合
     */
    List<DragTable> selectDragTableList(DragTable dragTable);

    /**
     * 新增动态表格
     * 
     * @param dragTable 动态表格
     * @return 结果
     */
    int insertDragTable(DragTable dragTable);

    /**
     * 修改动态表格
     * 
     * @param dragTable 动态表格
     * @return 结果
     */
    int updateDragTable(DragTable dragTable);

    /**
     * 删除动态表格
     * 
     * @param tId 动态表格主键
     * @return 结果
     */
    int deleteDragTableByTId(Long tId);

    /**
     * 批量删除动态表格
     * 
     * @param tIds 需要删除的数据主键集合
     * @return 结果
     */
    int deleteDragTableByTIds(List<Long> tIds);

    /**
     * 查询动态表格
     *
     * @param sqlKey 唯一标识
     * @return 动态表格
     */
    DragTable selectDragTableBySqlKey(String sqlKey);

    /**
     * 获取动态表单详情
     */
    DragTableVo selectDragTableVoByTId(Long tId);

    /**
     * 根据tableKey获取动态表格详情
     */
    DragTable selectDragTableByTableKey(String tableKey);

    /**
     * 根据tableKey批量获取动态表单详情
     */
    List<DragTableVo> selectDragTableTableKeys(List<String> tableKeys);

    /**
     * 根据tableKey批量获取动态表单详情
     */
    List<DragTableVo> selectDragTableTableKeysOrderby(List<String> tableKeys);

    /**
     * 根据tableKey批量删除动态表单详情
     */
    int deleteDragTableByTableKeys(List<String> tableKeys);
}
