package com.zkqy.system.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;
import com.zkqy.common.config.ZkqyConfig;
import com.zkqy.common.core.domain.AjaxResult;
import com.zkqy.common.utils.DateUtils;

import com.zkqy.common.utils.SecurityUtils;
import com.zkqy.common.utils.StringUtils;
import com.zkqy.system.entity.DragForm;
import com.zkqy.system.entity.DragTableForm;
import com.zkqy.system.mapper.CommonMapper;
import com.zkqy.system.mapper.DragFormMapper;
import com.zkqy.system.mapper.DragTableFormMapper;
import com.zkqy.system.service.IDragFormService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * 动态表单
 * Service业务层处理
 *
 * @author zkqy
 * @date 2023-07-12
 */
@Service
public class DragFormServiceImpl implements IDragFormService {

    @Autowired
    private DragFormMapper dragFormMapper;

    @Resource
    private DragTableFormMapper dragTableFormMapper;

    @Resource
    private CommonMapper commonMapper;

    /**
     * 查询动态表单
     *
     * @param fId 动态表单
     *            主键
     * @return 动态表单
     */
    @Override
    public DragForm selectDragFormByFId(Long fId) {
        return queryFormSelect(dragFormMapper.selectDragFormByFId(fId));
    }

    /**
     * 查询动态表单
     * 列表
     *
     * @param dragForm 动态表单
     * @return 动态表单
     */
    @Override
    public List<DragForm> selectDragFormList(DragForm dragForm) {
        return dragFormMapper.selectDragFormList(dragForm);
    }

    /**
     * 新增动态表单
     *
     * @param dragForm 动态表单
     * @return 结果
     */
    @Override
    @Transactional
    public AjaxResult insertDragForm(DragForm dragForm) {
        String fileName = ZkqyConfig.getProfile() + UUID.randomUUID().toString() + ".html";
        // 创建表单源文件
        createForm(fileName, dragForm.getDfHtmlTemplate());
        // 使用租户名 加uuid 定义表单文件名称
        dragForm.setDfFliePath(fileName);
        //校验dtId是否唯一
        if (dragTableFormMapper.selectDtIdCount(dragForm.getDtId()) > 0) {
            return AjaxResult.warn("该表格已绑定表单!");
        }
        dragForm.setCreateById(SecurityUtils.getUserId());
        dragForm.setCreateBy(SecurityUtils.getUsername());
        //新增表单
        dragFormMapper.insertDragForm(dragForm);
        //新增关联表
        DragTableForm dragTableForm = new DragTableForm();
        dragTableForm.setDfId(dragForm.getfId());
        dragTableForm.setDtId(dragForm.getDtId());
        dragTableFormMapper.insertDragTableForm(dragTableForm);
        return AjaxResult.success();
    }

    /**
     * 修改动态表单
     *
     * @param dragForm 动态表单
     * @return 结果
     */
    @Override
    public int updateDragForm(DragForm dragForm) {
        // 修改的时候删除表单源文件后重新创建
        delFileForm(dragForm.getfId());
        dragForm.setUpdateTime(DateUtils.getNowDate());
        dragForm.setUpdateById(SecurityUtils.getUserId());
        return dragFormMapper.updateDragForm(dragForm);
    }

    /**
     * 批量删除动态表单
     *
     * @param fIds 需要删除的动态表单
     *             主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteDragFormByFIds(Long[] fIds) {
        // 删除表单源文件
        for (Long fId : fIds) {
            delFileForm(fId);
            //删除表单表格关联表
            if (StringUtils.isNotNull(dragTableFormMapper.selectDragTableForm(fId))) {
                dragTableFormMapper.deleteDragTableFormByDfId(fId);
            }
        }
        return dragFormMapper.deleteDragFormByFIds(fIds);
    }

    /**
     * 删除动态表单
     * 信息
     *
     * @param fId 动态表单
     *            主键
     * @return 结果
     */
    @Override
    public int deleteDragFormByFId(Long fId) {

        // 删除表单源文件
        delFileForm(fId);
        return dragFormMapper.deleteDragFormByFId(fId);
    }

    @Override
    public DragForm selectDragFormBySqlKey(String sqlKey) {
        DragForm dragForm = dragFormMapper.selectDragFormBySqlKey(sqlKey);


//        //级联下拉框
//        //获取需要执行的sql集合
//        List<Object> sqlList = objectMapper.readValue(dragForm.getDfFormSql(), new TypeReference<List<Object>>() {});
//        //得到根节点数据
//        List<Map<String, Object>> rootList = commonMapper.executeSql(sqlList.get(0).toString());
//        for (Object sql: sqlList){
//            //跳过执行第一条sql，第一条为根节点
//            if(sqlList.indexOf(sql) == 0){
//                continue;
//            }
//
//
//
//        }
//        for (Map<String, Object> m : rootList){
//
//        }


        return queryFormSelect(dragForm);
    }

    /**
     * 查询当前表单的详细信息-》中前表单中所包含的下拉框
     */
    public DragForm queryFormSelect(DragForm dragForm) {
        ObjectMapper objectMapper = new ObjectMapper();
        Gson gson = new Gson();
        if (dragForm == null) {
            return dragForm;
        }
        //单个下拉框
        //获取要执行sql
        Map<String, Object> sqlMap = null;
        try {
            sqlMap = objectMapper.readValue(dragForm.getDfFormSql(), Map.class);
            //循环得到每一个sql语句返回结果
            for (Map.Entry<String, Object> entry : sqlMap.entrySet()) {
                List<Map<String, Object>> mapList = commonMapper.executeSql(String.valueOf(entry.getValue()));
                sqlMap.put(entry.getKey(), mapList);
            }
            dragForm.setDfFormSql(gson.toJson(sqlMap));
            return dragForm;
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 删除表单源文件
     *
     * @param fId
     */
    public void delFileForm(Long fId) {
        DragForm dragForm = dragFormMapper.selectDragFormByFId(fId);
        File file = new File(dragForm.getDfFliePath());
        if (file == null) return;
        if (file.delete()) {
            System.out.println("删除文件成功！");
        } else {
            System.out.println("删除文件失败！");
        }
    }

    /**
     * 创建表单源文件
     *
     * @param pathName 生成文件路径以及文件名称
     * @param text     写入文件中的内容
     */
    public void createForm(String pathName, String text) {
        // 新增的同时生成文件 生成路径从配置文件读取
        File file = new File(pathName);
        try {
            if (file.createNewFile()) { // 判断当前文件是否存在，
                System.out.println("创建文件成功！");
            } else {
                System.out.println("文件已经存在不需要重复创建");
            }
            FileWriter writer = new FileWriter(file);
            writer.write(text);
            writer.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}